/********************************************************************************************************
 * PROGRAM      : fgPlugins
 * DATE - TIME  : mardi 25 juillet 2006 - 18h57
 * AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox@gmail.com )
 * FILENAME     : fgLabel.cpp
 * LICENSE      : GPL
 * COMMENTARY   : 
 ********************************************************************************************************/
#include "fgLabel.h"
//
#include <QMouseEvent>
#include <QPainter>
//
fgLabel::fgLabel( QWidget* p )
	: QLabel( p ), _autoResize( true ), _toggleButton( false ), _toggled( false ),
	_data( "" ), _buttonData( "" ), _buttonTransparent( false )
{
	if ( toggleButton() )
		setToggled( toggled() );
}
//
void fgLabel::setImage( const QPixmap& p )
{
	// On affiche le bouton en arriere plan
	setPixmap( p );
	// Si le pixmap n'est pas nul et qu'il faut redimensionn� on le fait avec la taille du pixmap
	if ( !p.isNull() && _autoResize && size() != p.size() )
		resize( p.size() );
}
//
void fgLabel::mousePressEvent( QMouseEvent* e )
{
	// Si le pushed pixmap est non nul, on l'affiche
	if (
		_buttonTransparent
		||
		( !_buttonTransparent && !_buttonPushedPixmap.isNull() )
	  )
	{
		setImage( _buttonPushedPixmap );
	}
	QLabel::mousePressEvent( e );
}
//
void fgLabel::mouseReleaseEvent( QMouseEvent* e )
{
	// Si le widget est de type Button, on restore le pixmap normal
	if (
		_buttonTransparent
		||
		( !_buttonTransparent && !_buttonNormalPixmap.isNull() )
	  )
	{
		setImage( _buttonNormalPixmap );
	}
	// Si on relache la souris sur le widget, on emet les signaux
	if ( e->x() >= 0 && e->x() <= width() && e->y() >= 0 && e->y() <= height() )
	{
		emit clicked();
		emit clicked( _data );
		emit clicked( _data.toString() );
		emit clicked( _data.toInt() );
		if ( toggleButton() )
			setToggled( !toggled() );
	}
	QLabel::mouseReleaseEvent( e );
}
//
void  fgLabel::enterEvent( QEvent* e )
{
	emit mouseEnter();
	emit mouseEnter( _data );
	emit mouseEnter( _data.toString() );
	emit mouseEnter( _data.toInt() );
	QLabel::enterEvent( e );
}
//
void  fgLabel::leaveEvent( QEvent* e )
{
	emit mouseLeave();
	emit mouseLeave( _data );
	emit mouseLeave( _data.toString() );
	emit mouseLeave( _data.toInt() );
	QLabel::leaveEvent( e );
}
//
void fgLabel::paintEvent( QPaintEvent* e )
{
	QLabel::paintEvent( e );
	//  Si buttonData est non null on affiche
	if ( !_buttonData.isNull() )
	{
		QPainter paint( this );
		paint.drawText( rect(), /*Qt::AlignCenter,*/ _buttonData.toString() );
	}
}
//
const bool fgLabel::autoResize() const
{
	return _autoResize;
}
//
void fgLabel::setAutoResize( const bool b )
{
	if ( _autoResize != b )
		_autoResize = b;
}
//
const bool fgLabel::toggleButton() const
{
	return _toggleButton;
}
//
void fgLabel::setToggleButton( const bool b )
{
	if ( _toggleButton != b )
		_toggleButton = b;
}
//
const bool fgLabel::toggled() const
{
	return _toggled;
}
//
void fgLabel::setToggled( const bool b )
{
	if ( _toggled == b )
		return;
	_toggled = b;
	if ( _toggled )
		setImage( buttonPushedPixmap() );
	else
		setImage( buttonNormalPixmap() );
	emit toggled( _toggled );
}

//
const QVariant fgLabel::data() const
{
	return _data;
}
//
void fgLabel::setData( const QVariant& v )
{
	if ( _data != v )
		_data = v;
}
//
const QPixmap fgLabel::buttonNormalPixmap() const
{
	return _buttonNormalPixmap;
}
//
void fgLabel::setButtonNormalPixmap( const QPixmap& p )
{
	// Si le pixmap est different, on l enregistre et l affiche
	if ( _buttonNormalPixmap.toImage() != p.toImage() )
	{
		_buttonNormalPixmap = p;
		setImage( _buttonNormalPixmap );
	}
}
//
const QPixmap fgLabel::buttonPushedPixmap() const
{
	return _buttonPushedPixmap;
}
//
void fgLabel::setButtonPushedPixmap( const QPixmap& p )
{
	// Si le pixmap est different, on l enregistre
	if ( _buttonPushedPixmap.toImage() != p.toImage() )
		_buttonPushedPixmap = p;
}
//
const QVariant fgLabel::buttonData() const
{
	return _buttonData;
}
//
void fgLabel::setButtonData( const QVariant& v )
{
	if ( _buttonData != v )
		_buttonData = v;
}
//
const bool fgLabel::buttonTransparent() const
{
	return _buttonTransparent;
}
//
void fgLabel::setButtonTransparent( const bool b )
{
	if ( _buttonTransparent != b )
		_buttonTransparent = b;
}
