/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : dimanche 30 juillet 2006 - 13h04
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : misc.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "misc.h"
#include "Options.h"
#include "main.h"

#include <QTime>
#if QT_VERSION >= 0x040200
#include <QDesktopServices>
#endif
#ifdef Q_OS_WIN32
#include <qt_windows.h>
#endif
#include <QTextStream>
#include <QTemporaryFile>
#include <QProcess>
#include <QTimer>

#include <QDebug>

/** Encrypt string s **/
QString misc::encrypt( const QString& s )
{
	QString r;
	for ( int i = 0; i < s.length(); i++)
		r += ( s[i].unicode() < 0x20 ) ? s[i] : QChar( 0x1001F -s[i].unicode() );
	return r;
}
/** Decrypt string s **/
QString misc::decrypt( const QString& s )
{
	return ( encrypt( s ) );
}
/** return euro character **/
QString misc::euro()
{
	return QString::fromUtf8( "\342\202\254" ); // euro character
}
/** Wait m milliseconds **/
void misc::wait( int m )
{
	QTime t;
	t.start();
	while( t.elapsed() < m );
		QApplication::processEvents();
}
/** Execute the application associate with this file extension **/
bool misc::openUrl( const QUrl& url )
{
	QUrl u( url );
	if ( u.scheme().isEmpty() )
		u.setScheme( "file://" );
#if QT_VERSION >= 0x040200
	return QDesktopServices::openUrl( u );
#endif
	//
	bool b;
#if defined( Q_OS_WIN32 )
	QT_WA({
			b = ( int ) ShellExecute( UIMAIN->winId(), 0, (TCHAR*)u.toString().utf16(), 0, 0, SW_SHOWMAXIMIZED ) > 32;
		}, {
			b = ( int ) ShellExecuteA( UIMAIN->winId(), 0, u.toString().toLocal8Bit(), 0, 0, SW_SHOWMAXIMIZED ) > 32;
		});
#elif defined( Q_OS_MAC )
	b = QProcess::startDetached( QString( "open \"%1\"" ).arg( u.toString() ) );
#elif defined( Q_OS_UNIX )
	b =  true;
	// KDE
	if ( !QProcess::startDetached( QString( "kfmclient exec %1" ).arg( u.toString() ) ) )
		// FIREFOX
		if ( !QProcess::startDetached( QString( "firefox %1" ).arg( u.toString() ) ) )
			// MOZILLA
			if ( !QProcess::startDetached( QString( "mozilla %1" ).arg( u.toString() ) ) )
				// GALEON
				if ( !QProcess::startDetached( QString( "galeon %1" ).arg( u.toString() ) ) )
					b =  false;
#endif
	return b;
}
/** Return current free user **/
FreeAccount misc::identify()
{
	FreeAccount fa;
	fa.Login = Options::current()->value( "/FreeGo/Identify/Login" ).toString();
	fa.Password = misc::decrypt( Options::current()->value( "/FreeGo/Identify/Password" ).toString() );
	fa.Connexion = (ConnexionFreeBox)Options::current()->value( "/FreeGo/Identify/Connexion" ).toInt();
	return fa;
}
//
void misc::setCurrentFreeAccount( const QString& s )
{
	int i = freeAccountId( s );
	if ( i == -1 )
		return;
	Options::current()->beginReadArray( "FreeAccounts" );
		Options::current()->setArrayIndex( i );
		const QString l = Options::current()->value( "UserName" ).toString();
		const QString p = Options::current()->value( "Password" ).toString();
		const ConnexionFreeBox c = (ConnexionFreeBox)Options::current()->value( "Connexion" ).toInt();
	Options::current()->endArray();
	Options::current()->setValue( "/FreeGo/Identify/Login", l );
	Options::current()->setValue( "/FreeGo/Identify/Password", p );
	Options::current()->setValue( "/FreeGo/Identify/Connexion", c );
}
/** Return proxy informations **/
Proxy misc::proxy()
{
	Proxy p;
	p.Enabled = Options::current()->value( "/FreeGo/Proxy/Enabled", false ).toBool();
	p.Host = Options::current()->value( "/FreeGo/Proxy/Host" ).toString();
	p.Port = Options::current()->value( "/FreeGo/Proxy/Port", 3128 ).toInt();
	p.Login = Options::current()->value( "/FreeGo/Proxy/Login" ).toString();
	p.Password = decrypt( Options::current()->value( "/FreeGo/Proxy/Password" ).toString() );
	return p;
}
//
int misc::emailAccountId( const QString& account )
// Return account id or -1
{
	int size = Options::current()->beginReadArray( "MailAccounts" );
	int id = -1;
	for ( int i = 0; i < size; i++ )
	{
		Options::current()->setArrayIndex( i );
		if ( Options::current()->value( "UserName" ).toString().toLower() == account.toLower() )
		{
			id = i;
			break;
		}
	}
	Options::current()->endArray();
	return id;
}
//
void misc::emailAddAccount( const QString& account, const QString& password, const QString& server )
// Add account, or modify if UserName already exists
{
	// Enregistrement, si le compte existe il sera mis a jour
	int id = misc::emailAccountId( account );
	int size = Options::current()->value( "MailAccounts/size" ).toInt();
	if ( id == -1 )
	{
		id = size;
		size++;
	}
	Options::current()->beginWriteArray( "MailAccounts", size );
	Options::current()->setArrayIndex( id );
	Options::current()->setValue( "UserName", account );
	Options::current()->setValue( "Password", misc::encrypt( password ) );
	Options::current()->setValue( "Server", server );
	Options::current()->endArray();
}
//
void misc::emailRemoveAccount( const QString & account )
// Remove account
{
	int id = emailAccountId( account );
	if ( id == -1 )
		return;
	QString login, password, server;
	// Reading last entry
	int size = Options::current()->beginReadArray( "MailAccounts" );
	Options::current()->setArrayIndex( size -1 );
	login = Options::current()->value( "UserName" ).toString();
	password = Options::current()->value( "Password" ).toString();
	server = Options::current()->value( "Server" ).toString();
	Options::current()->endArray();
	// writing to id entry and remove last entry
	Options::current()->beginWriteArray( "MailAccounts", size -1 );
	Options::current()->setArrayIndex( id );
	Options::current()->setValue( "UserName", login );
	Options::current()->setValue( "Password", password );
	Options::current()->setValue( "Server", server );
	Options::current()->setArrayIndex( size -1 );
	Options::current()->remove( "" );
	Options::current()->endArray();
}
//
QStringList misc::emailAccounts()
// Return all accounts
{
	QStringList accounts;
	int size = Options::current()->beginReadArray( "MailAccounts" );
	for ( int i = 0; i < size; i++ )
	{

		Options::current()->setArrayIndex( i );
		accounts << Options::current()->value( "UserName" ).toString();

	}
	Options::current()->endArray();
	return accounts;
}
//
int misc::freeAccountId( const QString& account )
// Return account id or -1
{
	int size = Options::current()->beginReadArray( "FreeAccounts" );
	int id = -1;
	for ( int i = 0; i < size; i++ )
	{
		Options::current()->setArrayIndex( i );
		if ( Options::current()->value( "UserName" ).toString().toLower() == account.toLower() )
		{
			id = i;
			break;
		}
	}
	Options::current()->endArray();
	return id;
}
//
void misc::freeAddAccount( const QString& account, const QString& password, ConnexionFreeBox connexion )
// Add account, or modify if UserName already exists
{
	// Enregistrement, si le compte existse il sera mis a jour
	int id = misc::freeAccountId( account );
	int size = Options::current()->value( "FreeAccounts/size" ).toInt();
	if ( id == -1 )
	{
		id = size;
		size++;
	}
	Options::current()->beginWriteArray( "FreeAccounts", size );
	Options::current()->setArrayIndex( id );
	Options::current()->setValue( "UserName", account );
	Options::current()->setValue( "Password", misc::encrypt( password ) );
	Options::current()->setValue( "Connexion", connexion );
	Options::current()->endArray();
}
//
void misc::freeRemoveAccount( const QString & account )
// Remove account
{
	int id = freeAccountId( account );
	if ( id == -1 )
		return;
	QString login, password;
	ConnexionFreeBox connexion;
	// Reading last entry
	int size = Options::current()->beginReadArray( "FreeAccounts" );
	Options::current()->setArrayIndex( size -1 );
	login = Options::current()->value( "UserName" ).toString();
	password = Options::current()->value( "Password" ).toString();
	connexion = (ConnexionFreeBox)Options::current()->value( "Connexion" ).toInt();
	Options::current()->endArray();
	// writing to id entry and remove last entry
	Options::current()->beginWriteArray( "FreeAccounts", size -1 );
	Options::current()->setArrayIndex( id );
	Options::current()->setValue( "UserName", login );
	Options::current()->setValue( "Password", password );
	Options::current()->setValue( "Connexion", connexion );
	Options::current()->setArrayIndex( size -1 );
	Options::current()->remove( "" );
	Options::current()->endArray();
}
//
QStringList misc::freeAccounts()
// Return all account
{
	QStringList accounts;
	int size = Options::current()->beginReadArray( "FreeAccounts" );
	for ( int i = 0; i < size; i++ )
	{

		Options::current()->setArrayIndex( i );
		accounts << Options::current()->value( "UserName" ).toString();
	}
	Options::current()->endArray();
	return accounts;
}
//
QList<FreeAccount> misc::freeAccountsList()
{
	QList<FreeAccount> accounts;
	int size = Options::current()->beginReadArray( "FreeAccounts" );
	for ( int i = 0; i < size; i++ )
	{

		Options::current()->setArrayIndex( i );
		FreeAccount fa;
		fa.Login = Options::current()->value( "UserName" ).toString();
		fa.Password = misc::decrypt( Options::current()->value( "Password" ).toString() );
		fa.Connexion = (ConnexionFreeBox)Options::current()->value( "Connexion" ).toInt();
		accounts << fa;
	}
	Options::current()->endArray();
	return accounts;
}
//
void misc::openMailBox( const QString& l )
{
	QTemporaryFile* f = new QTemporaryFile( QString( "%1/IMPXXXXXX.html" ).arg( QDir::tempPath() ), qApp );
	if ( f->open() )
	{
		//openUrl( QUrl( "http://webmail.free-go.net/index.php?_action=logout" ) );
		if ( !QFile::exists( Options::settingsPath().append( "/webmail.png" ) ) )
			if ( !QFile::copy( ":/email/Pictures/email/webmail.png", Options::settingsPath().append( "/webmail.png" ) ) )
				UIMAIN->alert( QString( "can't create webmail png: %1" ).arg( Options::settingsPath().append( "/webmail.png" ) ) );
		//
		QTextStream t( f );
		QString u, p, s;
		//
		int i = emailAccountId( l );
		Options::current()->beginReadArray( "MailAccounts" );
			Options::current()->setArrayIndex( i );
			u = Options::current()->value( "UserName" ).toString();
			p = misc::decrypt( Options::current()->value( "Password" ).toString() );
			s = Options::current()->value( "Server" ).toString();
		Options::current()->endArray();
		//
		t << "<html>\n"
		<< "<head>\n"
		<< "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n"
		<< "<title>FreeGo - Chargement de votre boite mail, veuillez patienter</title>\n"
		<< "</head>\n"
		<< "<body bgcolor=#FFFFFF>\n"
		<< "<table border=\"0\" width=\"100%\" height=\"100%\">\n"
		<< "<tr>\n"
		<< "<td valign=\"middle\" align=\"center\">\n"
		<< "<img src=\"" << QDir::convertSeparators( Options::settingsPath().append( "/webmail.png" ) ).prepend( "file:///" ) << "\">\n"
		<< "</td>\n"
		<< "</table>\n"
		<< "<form name=\"form\" action=\"http://webmail.free-go.net/index.php\" method=\"post\">\n"
		<< "<input type=\"hidden\" name=\"_action\" value=\"login\" />\n"
		<< "<input type=\"hidden\" name=\"_user\" value=\"" << u.left( u.indexOf( '@' ) ) << "\" />\n"
		<< "<input type=\"hidden\" name=\"_pass\" value=\"" << p << "\" />\n"
		<< "<input type=\"hidden\" name=\"imageField\" value=\"\" />\n"
		<< "</form>\n"
		<< "<script>setTimeout( 'document.form.submit()', 5000)</script>\n"
		<< "</body>\n"
		<< "</html>\n";
		f->seek( 0 );
		openUrl( QUrl( f->fileName() ) );
		QTimer::singleShot( 30000, f, SLOT( deleteLater() ) );
	}
}
//
QByteArray misc::processLinks( const QByteArray& a, const QString& color )
{
	QString s( a );
	s.replace( QRegExp("(<a [^>]+>)" ), QString( "\\1<font color=\"%1\">" ).arg( color ) );
	s.replace( "</a>", "</font></a>" );
	return s.toAscii();
}
//
void misc::setAutoStart( bool b )
{
#ifdef Q_OS_WIN
	QSettings s( QSettings::UserScope, "Microsoft", "Windows" );
	s.remove( "/CurrentVersion/Run/FreeGo" );
	if ( b )
		s.setValue( "/CurrentVersion/Run/FreeGo", qApp->applicationFilePath() );
#else
	Q_UNUSED( b );
//	Alert( 0, "Cette fonctionnalit�n'est pas encore impl�ent� pour votre plate-forme.");
#endif
}
//
QHash<QString, QString> misc::parseHTMLTag( const QString& tag )
// parse a html tag, and return hash of key/value
{
	// for stocking values
	QHash<QString,QString> values;
	// regexp for getting attributes
	QRegExp valRx( "((\\w+)=(?:\\\"|')(([^\"|']+))(?:\\\"|'))+" );
	// getting attributes
	int pos = 0;
	while ( ( pos = valRx.indexIn( tag, pos ) ) != -1 )
	{
		values[ valRx.cap( 2 ).trimmed() ] = valRx.cap( 3 ).trimmed();
		pos += valRx.matchedLength() +1;
	}
	// get selected state
	if ( tag.contains( QRegExp( "\\bselected\\b" ) ) )
		values[ "#Selected" ] = "1";
	// getting content data for tag like <tag>content data</tag>
	const QString tagName = tag.mid( 1, tag.indexOf( ' ' ) -1 );
	QRegExp cntDataRx( QString( "(<%1.*>(.*)</%1>)" ).arg( tagName ) );
	pos = cntDataRx.indexIn( tag );
	if ( pos != -1 )
		values[ "#Content" ] = cntDataRx.cap( 2 ).trimmed();
	return values;
}
//
QList<QHash<QString, QString> > misc::parseHTMLSelect( const QString& select )
// parse select input options
{
	QList<QHash<QString, QString> > options;
	QRegExp rx( "<option.+</option>" );
	rx.setMinimal( true );
	int pos = 0;
	while ( ( pos = rx.indexIn( select, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get parse tag
		options << parseHTMLTag( rx.cap( 0 ).simplified() );
	}
	return options;
}
//
QString misc::toPlainText( const QString& html )
// convert html string to plain text
{
	QTextDocument doc;
	doc.setHtml( html );
	return doc.toPlainText();
}
//
QList<QStringList> misc::parseHtmlTable( const QString& table, bool plainText )
// parse a html table and return the result
{
	QList<QStringList> mResult;
	// td regexp
	QRegExp tdrx( "<td.*>((.*))</td>" );
	tdrx.setMinimal( true );
	// get tr tags
	QRegExp rx( "<tr.*>(.*)</tr>" );
	rx.setMinimal( true );
	int rxpos = 0;
	while ( ( rxpos = rx.indexIn( table, rxpos ) ) != -1 )
	{
		// increment pos
		rxpos += rx.matchedLength();
		// get tr tag
		QString tr = rx.cap( 0 ).simplified();
		// get td tags
		int tdrxpos = 0;
		QStringList row;
		while ( ( tdrxpos = tdrx.indexIn( tr, tdrxpos ) ) != -1 )
		{
			// increment pos
			tdrxpos += tdrx.matchedLength();
			// get tr tag
			QString td = tdrx.cap( 1 ).simplified();
			if ( plainText )
				td = misc::toPlainText( td );
			// add value
			row << td;
		}
		if ( !row.isEmpty() )
			mResult << row;
	}
	return mResult;
}
//
QTextEdit* misc::debugInTextEdit( const QString& string, QTextEdit* edit )
{
	if ( !edit )
		edit = new QTextEdit;
	edit->insertPlainText( string );
	return edit;
}
