/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : jeudi 24 aot 2006 - 18h51
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UIMonitor.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIMonitor.h"
#include "Options.h"
//
#include <QTimerEvent>
#include <QLibrary>
#include <QPainter>
#include <QFile>
#include <QMessageBox>
#include <QDir>
//
QPointer<UIMonitor> UIMonitor::mSelf = 0L;
//
UIMonitor* UIMonitor::self( QWidget* p  )
{
	if ( !mSelf )
		mSelf = new UIMonitor( p );
	return mSelf;
}
//
UIMonitor::UIMonitor( QWidget* p )
	: fgDialog( p ),
	mTimerId( 0 ), mIsStarted( false ),
	mFirst( true ), mJaugeMax( 143 ),
	mDLMax( 5.0 ), mUPMax( 5.0 ),  // Default 56k, it will increase itself for best connection
	mDownload( -1 ), mUpload( -1 ),
	down_h1( 0 ), down_h2( 0 ), up_h1( 0 ), up_h2( 0 )
{
	setupUi( this );
	fixMacWidgets();
	if ( Options::current()->value( "/FreeGo/Options/Monitor/AlwaysOnTop", false ).toBool() )
		setWindowFlags( windowFlags() | Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint );
	setWindowOpacity( Options::current()->value( "/FreeGo/Options/Monitor/Opacity", 100.0 ).toDouble() / 100.0 );
}
//
void UIMonitor::showEvent( QShowEvent* e )
{
	fgDialog::showEvent( e );
	if ( !start() )
		deleteLater();
}
//
void UIMonitor::closeEvent( QCloseEvent* e )
{
	stop();
	fgDialog::closeEvent( e );
}
//
void UIMonitor::timerEvent( QTimerEvent* e )
{
	if ( !mIsStarted )
		return;
	if ( e->timerId() == mTimerId )
	{
		down_h1 = down_h2;
		up_h1 = up_h2;
#if defined( Q_OS_WIN32 )
		uint x = 0;
		down_h2 = up_h2 = 0;         
		getiftable = table( buffer, &buf_size, TRUE);
		while ( buffer && x <= buffer->dwNumEntries )
		{
			down_h2 += buffer->table[x].dwInOctets;
			up_h2 += buffer->table[x].dwOutOctets;
			x++;
		}
		mUpload = ( float ) ( up_h2 -up_h1 ) / 1010;
		mDownload = ( float ) ( down_h2 -down_h1 ) / 1010;
#elif defined( Q_OS_UNIX )
		mUpload = mDownload = 0;
		down_h2 = up_h2 = 0;         
		// Comme on ne connait pas l'interface active, on parcours tous les r�ertoires
		// et on va utiliser celui dont le d�it varie.
		QDir dir( "/sys/class/net/" );
		foreach ( QString s, dir.entryList( QDir::NoDotAndDotDot ) )
		{
			if( s.contains( "." ) || s == "lo" )
				continue;
			QFile ul( QString( "/sys/class/net/%1/statistics/rx_bytes" ).arg( s ) );
			if ( ul.open( QFile::ReadOnly | QFile::Text ) )
			{
				char buf[1024];
				ul.readLine( buf, sizeof( buf ) );
				down_h2 += QString( buf ).toFloat();
			}
			else
				qWarning( "can t open ul" );
			QFile dl( QString( "/sys/class/net/%1/statistics/tx_bytes" ).arg( s ) );
			if ( dl.open( QFile::ReadOnly | QFile::Text ) )
			{
				char buf[1024];
				dl.readLine( buf, sizeof( buf ) );
				up_h2 += QString( buf ).toFloat();
			}
			else
				qWarning( "can t open dl" );
			if( ( up_h2 && down_h2 ) && up_h2 > up_h1 && down_h2 > down_h1 )
			{
				/*
				Si le debit a chang� je consid�e que c'est l'interface active.
				Je quitte par break apr� avoir renseign�mUpload et mDownload
				*/
				mUpload = ( float ) ( up_h2 -up_h1 ) / 1010;
				mDownload = ( float ) ( down_h2 -down_h1 ) / 1010;
				break;
			}
		}
#endif
		//
		updateGraph();
	}
}
//
bool UIMonitor::start()
{
	mIsStarted = false;
#if defined( Q_OS_WIN32 )
	buffer = ( MIB_IFTABLE* ) malloc( sizeof( MIB_IFTABLE ) );
	buf_size = sizeof( MIB_IFTABLE );
	table = (GetIfTable)QLibrary::resolve( "iphlpapi", "GetIfTable" );
	if( table )
		getiftable = table( buffer, &buf_size, TRUE);
	else
		return mIsStarted;
	//
	buffer = ( MIB_IFTABLE* ) malloc( buf_size );
	mTimerId = startTimer( 1000 );
	mIsStarted = true;
#elif defined( Q_OS_UNIX )
	mTimerId = startTimer( 1000 );
	mIsStarted = true;
#endif
	return mIsStarted;
}
//
bool UIMonitor::stop()
{
	if ( mIsStarted )
	{
		mIsStarted = false;
		killTimer( mTimerId );
	}
	return true;
}
//
void UIMonitor::updateGraph()
{
	if ( !mIsStarted )
		return;
	// Waiting first values ( 0, 0 )
	if ( mFirst )
	{
		mFirst = false;
		return;
	}
	//
	if ( mDownload > mDLMax )
		mDLMax = mDownload;
	if ( mUpload > mUPMax )
		mUPMax = mUpload;
	if( mDownload > 1023 )
		lDLRate->setText( QString( "%1 Mo/s" ).arg( QString::number( mDownload / 1024.0, 'f', 2 ) ) );
	else
		lDLRate->setText( QString( "%1 Ko/s" ).arg( QString::number( mDownload, 'f', 2 ) ) );
	if( mUpload > 1023 )
		lUPRate->setText( QString( "%1 Mo/s" ).arg( QString::number( mUpload / 1024.0, 'f', 2 ) ) );
	else
		lUPRate->setText( QString( "%1 Ko/s" ).arg( QString::number( mUpload, 'f', 2 ) ) );
	int wDL, wUP;
	wDL = ( int ) ( ( mDownload / mDLMax ) * mJaugeMax );
	wUP = ( int ) ( ( mUpload / mUPMax ) * mJaugeMax );
	lDLJauge->resize( wDL, lDLJauge->height() );
	lUPJauge->resize( wUP, lUPJauge->height() );
}
