/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : dimanche 01 octobre 2006 - 16h01
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UIWifiRooter.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIWifiRooter.h"
#include "main.h"
#include "misc.h"

#include <QSettings>

#include <math.h>

QPointer<UIWifiRooter> UIWifiRooter::mSelf = 0L;

UIWifiRooter* UIWifiRooter::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIWifiRooter( p );
	return mSelf;
}

UIWifiRooter::UIWifiRooter( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	rwPorts->setType( UIRooterWidget::Ports );
	rwPlages->setType( UIRooterWidget::Plages );
	rwDHCP->setType( UIRooterWidget::Macs );
	fixMacWidgets();
	
	// connections
	// menu
	QList<fgLabel*> buttons = findChildren<fgLabel*>( QRegExp( "fgMenu" ) );
	foreach ( fgLabel* button, buttons )
		connect( button, SIGNAL( clicked( const int ) ), this, SLOT( fgMenu_clicked( const int ) ) );
	// ip
	connect( leIpFreebox3, SIGNAL( textChanged( const QString& ) ), this, SLOT( setIp3( const QString& ) ) );
	// rooter
	connect( fgSaveRooter1, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	connect( fgSaveRooter2, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	connect( fgSaveRooter3, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	connect( fgSaveRooter4, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	// wifi
	connect( fgSaveWifi1, SIGNAL( clicked() ), this, SLOT( fgSaveWifi() ) );
	connect( fgSaveWifi2, SIGNAL( clicked() ), this, SLOT( fgSaveWifi() ) );
	
	// login
	REQUESTER->login( this, "readLogin" );
}

void UIWifiRooter::fgMenu_clicked( int i )
{
	QList<fgLabel*> buttons = findChildren<fgLabel*>( QRegExp( "fgMenu" ) );
	foreach ( fgLabel* button, buttons )
	{
		button->setToggled( button->data().toInt() == i );
		if ( sender() && button->data().toInt() == i )
			button->setToggled( false );
	}
	swPages->setCurrentIndex( i );
}

void UIWifiRooter::setIp3( const QString& value )
{
	leDebutDHCP3->setText( value );
	leIpDmz3->setText( value );
	leFinDHCP3->setText( value );
	leIpFreeplayer3->setText( value );
	rwPorts->setIp3( value );
	rwPlages->setIp3( value );
	rwDHCP->setIp3( value );
}

void UIWifiRooter::readLogin( QSettings*, bool error )
{
	if ( error )
	{
		UIMAIN->alert( "Impossible de vous identifiez sur free, annulation" );
		deleteLater();
		return;
	}
	requestRooter( QByteArray(), false );
	requestWifi( QByteArray(), false );
	fgMenu_clicked( 0 );
}

void UIWifiRooter::requestRooter( const QByteArray& buffer, bool error )
{
	// check buffer
	if ( !buffer.isEmpty() && !buffer.contains( "Valeurs mises &agrave; jour avec succ&egrave;s" ) )
		error = true;
	if ( error )
	{
		QString ba = QString::fromUtf8( buffer );
		int pos = ba.indexOf( "fbxcfgwww_result" );
		pos = ba.indexOf( "<p>", pos ) +3;
		int end = ba.indexOf( "</p>", pos );
		ba = ba.mid( pos, end -pos ).simplified();
		
		QString msg = "Une erreur est survenue lors de la modification (Rooter)";
		if ( !ba.isEmpty() )
			msg.append( "\nErreur Free: " +ba );
		UIMAIN->alert( msg );
		pConfigRooter->setEnabled( true );
		pPortsRedirect->setEnabled( true );
		pPortsPlage->setEnabled( true );
		pBaudsDHCP->setEnabled( true );
		return;
	}
	// request rooter configuration
	REQUESTER->requestUrl( this, "readRooter", REQUESTER->freeBaseUrl().append( "admin/fbxcfg/fbxcfg.pl?&tpl=router" ), "GET", pHttpRequest::rtByteArray );
	// disable pages
	pConfigRooter->setEnabled( false );
	pPortsRedirect->setEnabled( false );
	pPortsPlage->setEnabled( false );
	pBaudsDHCP->setEnabled( false );
}

void UIWifiRooter::readRooter( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( "Une erreur est survenue lors de la recuperation de la configuration Rooter" );
		return;
	}
	
	// parse
	QString ba = QString::fromLatin1( buffer );
	
	QString key = "<form";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "</form>";
	pos = ba.indexOf( key );
	ba = ba.left( pos +key.length() ).simplified();
	
	// ports
	QStringList redir_ports; // redir_port_new
	QStringList redir_protos; // redir_proto_new
	QStringList redir_dests; // redir_dest_new
	QStringList redir_dest_ports; // redir_dest_port_new
	
	// plages
	QStringList mredir_ports; // mredir_port_new
	QStringList mredir_dest_ports; // mredir_dest_port_new
	QStringList mredir_protos; // mredir_proto_new
	QStringList mredir_dests; // mredir_dest_new
	
	// macs
	QStringList lease_ips; // lease_ip_new
	QStringList lease_macs; // lease_mac_new
	
	// parse inputs
	QRegExp rx( "<input.*>" );
	rx.setMinimal( true );
	pos = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get input tag
		const QString tag = rx.cap( 0 ).simplified();
		const QHash<QString, QString> values = misc::parseHTMLTag( tag );
		const QString name = values.value( "name" );
		const bool isChecked = ( values.value( "type" ) == "checkbox" || values.value( "type" ) == "radio" ) && tag.contains( "checked", Qt::CaseInsensitive );
		const QString value = values.value( "value" );
		//checked
		if ( name == "router_active" )
			cbActivation->setToggled( isChecked );
		else if ( name == "lan_pfx" )
			leIpFreebox3->setText( value );
		else if ( name == "lan_ip" )
			leIpFreebox4->setText( value );
		else if ( name == "dhcp_active" )
			cbDHCPActivation->setToggled( isChecked );
		else if ( name == "dhcp_start" )
			leDebutDHCP4->setText( value );
		else if ( name == "dhcp_end" )
			leFinDHCP4->setText( value );
		else if ( name == "dmz_host" )
			leIpDmz4->setText( value );
		else if ( name == "freeplayer" )
			leIpFreeplayer4->setText( value );
		else if ( name == "answer_ping" )
			cbReponsePing->setToggled( isChecked );
		else if ( name == "proxy_wol" )
			cbWOL->setToggled( isChecked );
		// ports
		else if ( name.startsWith( "redir_port_" ) && !name.endsWith( "_new" ) )
			redir_ports << value;
		else if ( name.startsWith( "redir_dest_port_" ) && !name.endsWith( "_new" ) )
			redir_dest_ports << value;
		else if ( name.startsWith( "redir_dest_" ) && !name.endsWith( "_new" ) )
			redir_dests << value;
		// plages
		else if ( name.startsWith( "mredir_port_" ) && !name.endsWith( "_new" ) )
			mredir_ports << value;
		else if ( name.startsWith( "mredir_dest_port_" ) && !name.endsWith( "_new" ) )
			mredir_dest_ports << value;
		else if ( name.startsWith( "mredir_dest_" ) && !name.endsWith( "_new" ) )
			mredir_dests << value;
		// macs
		else if ( name.startsWith( "lease_ip_" ) && !name.endsWith( "_new" ) )
			lease_ips << value;
		else if ( name.startsWith( "lease_mac_" ) && !name.endsWith( "_new" ) )
			lease_macs << value;
	}
	
	// parse selects
	// ports
	rx = QRegExp( "<select name=\"redir_proto_\\d+\">.+</select>" );
	rx.setMinimal( true );
	pos = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get options
		const QList<QHash<QString, QString> > options = misc::parseHTMLSelect( rx.cap( 0 ).simplified() );
		for ( int i = 0; i < options.count(); i++ )
		{
			const QHash<QString, QString> option = options.at( i );
			if ( option.value( "#Selected" ).toInt() == 1 )
				redir_protos << option.value( "value" );
		}
	}
	// plages
	rx = QRegExp( "<select name=\"mredir_proto_\\d+\">.+</select>" );
	rx.setMinimal( true );
	pos = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get options
		const QList<QHash<QString, QString> > options = misc::parseHTMLSelect( rx.cap( 0 ).simplified() );
		for ( int i = 0; i < options.count(); i++ )
		{
			const QHash<QString, QString> option = options.at( i );
			if ( option.value( "#Selected" ).toInt() == 1 )
				mredir_protos << option.value( "value" );
		}
	}
	
	// create datas for AbstractRooterWidgets
	// ports
	QStringList ports_datas;
	for ( int i = 0; i < redir_ports.count(); i++ )
		ports_datas << QString( "%1|%2|%3|%4" )
			.arg( redir_ports.value( i ) )
			.arg( redir_protos.value( i ) )
			.arg( redir_dests.value( i ) )
			.arg( redir_dest_ports.value( i ) );
	// plages
	QStringList plages_datas;
	for ( int i = 0; i < mredir_ports.count(); i++ )
		plages_datas << QString( "%1|%2|%3|%4" )
			.arg( mredir_ports.value( i ) )
			.arg( mredir_dest_ports.value( i ) )
			.arg( mredir_protos.value( i ) )
			.arg( mredir_dests.value( i ) );
	// macs
	QStringList macs_datas;
	for ( int i = 0; i < lease_ips.count(); i++ )
		macs_datas << QString( "%1|%2" )
			.arg( lease_ips.value( i ) )
			.arg( lease_macs.value( i ) );
	
	// set AbstractRooterWidgets datas
	rwPorts->setDatas( ports_datas );
	rwPlages->setDatas( plages_datas );
	rwDHCP->setDatas( macs_datas );
	
	// enable pages
	pConfigRooter->setEnabled( true );
	pPortsRedirect->setEnabled( true );
	pPortsPlage->setEnabled( true );
	pBaudsDHCP->setEnabled( true );
}

void UIWifiRooter::fgSaveRooter()
{
	// post change
	QString mPost = REQUESTER->freeBaseUrl().append( "admin/fbxcfg/fbxcfg.pl?tpl=router&action=update&Envoyer" );
	mPost += QString( "&router_active=%1" ).arg( cbActivation->toggled() ? 1 : 0 );
	mPost += QString( "&lan_pfx=%1" ).arg( leIpFreebox3->text() );
	mPost += QString( "&lan_pfx_copy=%1" ).arg( leIpFreebox3->text() );
	mPost += QString( "&lan_ip=%1" ).arg( leIpFreebox4->text() );
	mPost += QString( "&dhcp_active=%1" ).arg( cbDHCPActivation->toggled() ? 1 : 0 );
	mPost += QString( "&dhcp_start=%1" ).arg( leDebutDHCP4->text() );
	mPost += QString( "&dhcp_end=%1" ).arg( leFinDHCP4->text() );
	mPost += QString( "&dmz_host=%1" ).arg( leIpDmz4->text() );
	mPost += QString( "&freeplayer=%1" ).arg( leIpFreeplayer4->text() );
	mPost += QString( "&answer_ping=%1" ).arg( cbReponsePing->toggled() ? 1 : 0 );
	mPost += QString( "&proxy_wol=%1" ).arg( cbWOL->toggled() ? 1 : 0 );
	// ports
	const QStringList ports_datas = rwPorts->datas();
	for ( int i = 0; i < ports_datas.count(); i++ )
	{
		const QStringList datas = ports_datas.at( i ).split( "|", QString::SkipEmptyParts );
		mPost += QString( "&redir_port_%1=%2" ).arg( i ).arg( datas.value( 0 ) );
		mPost += QString( "&redir_proto_%1=%2" ).arg( i ).arg( datas.value( 1 ) );
		mPost += QString( "&redir_dest_%1=%2" ).arg( i ).arg( datas.value( 2 ) );
		mPost += QString( "&redir_dest_port_%1=%2" ).arg( i ).arg( datas.value( 3 ) );
	}
	// plages
	const QStringList plages_datas = rwPlages->datas();
	for ( int i = 0; i < plages_datas.count(); i++ )
	{
		const QStringList datas = plages_datas.at( i ).split( "|", QString::SkipEmptyParts );
		mPost += QString( "&mredir_port_%1=%2" ).arg( i ).arg( datas.value( 0 ) );
		mPost += QString( "&mredir_dest_port_%1=%2" ).arg( i ).arg( datas.value( 1 ) );
		mPost += QString( "&mredir_proto_%1=%2" ).arg( i ).arg( datas.value( 2 ) );
		mPost += QString( "&mredir_dest_%1=%2" ).arg( i ).arg( datas.value( 3 ) );
	}
	// plages
	const QStringList macs_datas = rwDHCP->datas();
	for ( int i = 0; i < macs_datas.count(); i++ )
	{
		const QStringList datas = macs_datas.at( i ).split( "|", QString::SkipEmptyParts );
		mPost += QString( "&lease_ip_%1=%2" ).arg( i ).arg( datas.value( 0 ) );
		mPost += QString( "&lease_mac_%1=%2" ).arg( i ).arg( datas.value( 1 ) );
	}

	// send request
	REQUESTER->requestUrl( this, "requestRooter", mPost, "GET", pHttpRequest::rtByteArray );
	
	// disable pages
	pConfigRooter->setEnabled( false );
	pPortsRedirect->setEnabled( false );
	pPortsPlage->setEnabled( false );
	pBaudsDHCP->setEnabled( false );
}

void UIWifiRooter::requestWifi( const QByteArray& buffer, bool error )
{
	// check buffer
	if ( !buffer.isEmpty() && !buffer.contains( "Valeurs mises &agrave; jour avec succ&egrave;s" ) )
		error = true;
	if ( error )
	{
		QString ba = QString::fromUtf8( buffer );
		int pos = ba.indexOf( "fbxcfgwww_result" );
		pos = ba.indexOf( "<p>", pos ) +3;
		int end = ba.indexOf( "</p>", pos );
		ba = ba.mid( pos, end -pos ).simplified();
		
		QString msg = "Une erreur est survenue lors de la modification (Wifi)";
		if ( !ba.isEmpty() )
			msg.append( "\nErreur Free: " +ba );
		UIMAIN->alert( msg );
		pConfigWifi->setEnabled( true );
		pWifiSecurity->setEnabled( true );
		return;
	}
	// request wifi configuration
	REQUESTER->requestUrl( this, "readWifi", REQUESTER->freeBaseUrl().append( "admin/fbxcfg/fbxcfg.pl?&tpl=wifi" ), "GET", pHttpRequest::rtByteArray );
	pConfigWifi->setEnabled( false );
	pWifiSecurity->setEnabled( false );
}

void UIWifiRooter::readWifi( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( "Une erreur est survenue lors de la recuperation de la configuration Wifi" );
		return;
	}
	
	// parse
	QString ba = QString::fromLatin1( buffer );
	
	QString key = "<form";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "</form>";
	pos = ba.indexOf( key );
	ba = ba.left( pos +key.length() ).simplified();
	
	// parse inputs
	QRegExp rx( "<input.*>" );
	rx.setMinimal( true );
	pos = 0;
	while ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// increment pos
		pos += rx.matchedLength();
		// get input tag
		const QString tag = rx.cap( 0 ).simplified();
		const QHash<QString, QString> values = misc::parseHTMLTag( tag );
		const QString name = values.value( "name" );
		const bool isChecked = ( values.value( "type" ) == "checkbox" || values.value( "type" ) == "radio" ) && tag.contains( "checked", Qt::CaseInsensitive );
		const QString value = values.value( "value" );
		//checked
		if ( name == "wifi_active" )
			cbActivationService->setToggled( isChecked );
		else if ( name == "wifi_channel_auto" )
			cbAutomaticCanal->setToggled( isChecked );
		else if ( name == "wifi_ssid" )
			leNomReseau->setText( value );
		else if ( name == "wifi_ssid_hide" )
			cbHiddenNetwork->setToggled( isChecked );
		else if ( name == "wifi_key_type" && isChecked )
		{
			if ( value == "1" )
				rbWep->setChecked( true );
			else if ( value == "3" )
				rbWpaTkip->setChecked( true );
			else if ( value == "4" )
				rbWpaAesCcmp->setChecked( true );
			else if ( value == "2" )
				rbWpaTkipAes->setChecked( true );
		}
		else if ( name == "wifi_key" )
			leClef->setText( value );
	}
	
	rx = QRegExp( "<option value=\"\\d\"\\s+selected\\s+>.*</option>" );
	rx.setMinimal( true );
	if ( ( pos = rx.indexIn( ba, pos ) ) != -1 )
	{
		// get input tag
		const QString tag = rx.cap( 0 ).simplified();
		const QHash<QString, QString> values = misc::parseHTMLTag( tag );
		const int id = values.value( "value" ).toInt();
		cbCanal->setCurrentIndex( id -1 );
	}
	
	pConfigWifi->setEnabled( true );
	pWifiSecurity->setEnabled( true );
}

void UIWifiRooter::on_fgGenerateKey_clicked()
// wep/wpa key generation, algo stolen from free.fr javascript script
{
	int wifiType = -1;
	if ( rbWep->isChecked() )
		wifiType = 1;
	else if ( rbWpaTkip->isChecked() )
		wifiType = 3;
	else if ( rbWpaAesCcmp->isChecked() )
		wifiType = 4;
	else if ( rbWpaTkipAes->isChecked() )
		wifiType = 2;
	
	QString chars;
	int len;
	switch ( wifiType )
	{
		case 1: // wep
			chars = "0123456789ABCDEF";
			len = 10; // 10 or 26
			break;
		case 2: // wpa
		case 3:
		case 4:
			chars = " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_\\-;.?,:!%*^&~{}()|@=+<>";
			len = 63;
			break;
		default:
			return;
			break;
	}
	
	QString key;
	for ( int i = 0; i < len; i++ )
	{
		int rnd = qrand() %( chars.length() );
		key += chars[ rnd ];
	}
	
	leClef->setText( key );
}

void UIWifiRooter::fgSaveWifi()
{
	if ( rbWep->isChecked() )
	{
		// wep
		if ( !QRegExp( "[A-F|0-9]{10}" ).exactMatch( leClef->text() ) && !QRegExp( "[A-F|0-9]{26}" ).exactMatch( leClef->text() ) )
		{
			UIMAIN->alert( "Votre clef WEP n'est pas valide." );
			return;
		}
	}
	else
	{
		// wpa
		if ( !QRegExp( ".{8,63}" ).exactMatch( leClef->text() ) )
		{
			UIMAIN->alert( "Votre clef WPA n'est pas valide." );
			return;
		}
	}
	
	// get wifi type
	int wifiType = -1;
	if ( rbWep->isChecked() )
		wifiType = 1;
	else if ( rbWpaTkip->isChecked() )
		wifiType = 3;
	else if ( rbWpaAesCcmp->isChecked() )
		wifiType = 4;
	else if ( rbWpaTkipAes->isChecked() )
		wifiType = 2;
	
	// check wifi type
	if ( wifiType == -1 )
	{
		UIMAIN->alert( "Vous devez choisir un type de protection." );
		return;
	}
	
	// post change
	QString mPost = REQUESTER->freeBaseUrl().append( "admin/fbxcfg/fbxcfg.pl?tpl=wifi&action=update&Envoyer" );
	mPost += QString( "&wifi_active=%1&wifi_channel=%2&wifi_ssid=%3&wifi_key_type=%4&wifi_key=%5&wifi_channel_auto=%6&wifi_ssid_hide=%7" )
		.arg( cbActivationService->toggled() )
		.arg( cbCanal->currentIndex() +1 )
		.arg( leNomReseau->text() )
		.arg( wifiType )
		.arg( leClef->text() )
		.arg( cbAutomaticCanal->toggled() )
		.arg( cbHiddenNetwork->toggled() );
	REQUESTER->requestUrl( this, "requestWifi", mPost, "GET", pHttpRequest::rtByteArray );
	
	pConfigWifi->setEnabled( false );
	pWifiSecurity->setEnabled( false );
}
